;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
;; Include all function entry points as global symbols for the compiler.
.include "main.h.s"
.include "rsx.h.s" ;carga punto de entrada global de "rsx.s"
.include "macros.h.s" ;macros para las instrucciones que faltan

;;
;; Start of _DATA area 
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA
;; Direcciones RAM de sprites en SIMBUFFER.BIN
   youwin_sp::   .db #0x5B,#0x56     ;Byte Bajo primero
   you_sp::      .db #0x23,#0x56
   gameover_sp:: .db #0x2B,#0x55

;; Dirección de almacenamiento donde guardo el Número del Display que quiero imprimir (0-9)
   digito_display:: .db #0

;; puntero a la memoria de video para imprimir los sprites del Menú (interno)   
   mem_ptr::   .db #0   

;; Redefinición de teclas. Los 2 primeros bytes son LB, HB de la Constante ASM Keyboard. El 3º es el FKC.
   tecla_arriba::    .db #0,#0,#0
   tecla_abajo::     .db #0,#0,#0
   tecla_izquierda:: .db #0,#0,#0
   tecla_derecha::   .db #0,#0,#0
   tecla_repetir::   .db #0,#0,#0
   tecla_maspuntos:: .db #0,#0,#0   
   fkc_up::    .db #0 ;Basic graba FKC en el 3er byte de "tecla_arriba", y lo extraigo aqui
   fkc_down::  .db #0
   fkc_left::  .db #0
   fkc_right:: .db #0
   fkc_rep::   .db #0
   fkc_more::  .db #0   



_constantes_keycode_asm::
;Matriz Keycode Constant Keyboard ASM (2 bytes, en orden desde tecla 0-79, en grupos de 10)
;El índice del array o contador del bucle, indica el FKC (0-79). Se almacenan Byte Bajo 1º.
.dw 0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000, 0x8000, 0x0101, 0x0201
.dw 0x0401, 0x0801, 0x1001, 0x2001, 0x4001, 0x8001, 0x0102, 0x0202, 0x0402, 0x0802
.dw 0x1002, 0x2002, 0x4002, 0x8002, 0x0103, 0x0203, 0x0403, 0x0803, 0x1003, 0x2003
.dw 0x4003, 0x8003, 0x0104, 0x0204, 0x0404, 0x0804, 0x1004, 0x2004, 0x4004, 0x8004
.dw 0x0105, 0x0205, 0x0405, 0x0805, 0x1005, 0x2005, 0x4005, 0x8005, 0x0106, 0x0206
.dw 0x0406, 0x0806, 0x1006, 0x2006, 0x4006, 0x8006, 0x0107, 0x0207, 0x0407, 0x0807
.dw 0x1007, 0x2007, 0x4007, 0x8007, 0x0108, 0x0208, 0x0408, 0x0808, 0x1008, 0x2008
.dw 0x4008, 0x8008, 0x0109, 0x0209, 0x0409, 0x0809, 0x1009, 0x2009, 0x4009, 0x8009

;;Tecla pulsada Redefinida
tecla_pulsada:: .db 128 ;grabar el FKC (0-79) de cpct_isKeyPressed. Por defecto 128. Error 255.
tecla_pul_char:: .db 0 ;carácter ASCII asociado al FKC de la tecla pulsada
contador:: .db 0 ;contador del Array (0-79) que coincide con el FKC de la tecla pulsada
valorArray:: .db 0, 0 ;Se extrae 1 Cte. keycode asm de teclado, por ej. 8008, byte alto 1º
tecla_redef:: .db 0   ;Desde BASIC le digo a CM qué tecla estoy redefiniendo (1-4)


;;
;; Start of _CODE area
;; 
.area _CODE
;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;
_main::
     
   call _transfer

   call _RSX
   ;call _extraer_FKCs
     
   
ret


;***********************************************************************
;              ESCANEAR TECLADO Y ENVIAR TECLA PULSADA
;***********************************************************************
_escanear_teclado::
 ;******************* Escanear teclado y comprobar tecla pulsada *******************
 ;;Leer el teclado y comprobar si alguna de las teclas del Amstrad F1, F3, F4 y F6 
 ;;están pulsadas.   
   ;;Reads the status of keyboard and joysticks and stores it in the 10 bytes reserved 
   ;;as cpct_keyboardStatusBuffer.  
   call cpct_scanKeyboard_asm

   ;;cpct_isKeyPressed
   ;;Comprueba si una tecla concreta (Keycode asm 0xNNNN) está presionada  o no
   ;;Return value:
   ;;<u8>	falso (0, si no está presionado) o verdadero (> 0, si está presionado).
   ;;Flag Output Status: Z = 1 (Key NOT Pressed) / 0 (Key Pressed)
   ld hl,(tecla_arriba)               ;HL carga 2 bytes Keycode asm por ej. 0x2001=constante #Key_F1
   call cpct_isKeyPressed_asm
   jr nz, .arriba

   ld hl,(tecla_abajo)
   call cpct_isKeyPressed_asm
   jr nz, .abajo

   ld hl,(tecla_izquierda)
   call cpct_isKeyPressed_asm
   jr nz, .izquierda

   ld hl,(tecla_derecha)
   call cpct_isKeyPressed_asm
   jr nz, .derecha

   ld hl,(tecla_repetir)
   call cpct_isKeyPressed_asm
   jr nz, .repetir

   ld hl,(tecla_maspuntos)
   call cpct_isKeyPressed_asm
   jr nz, .maspuntos

  
   ld a,#255	; Ninguna corresponde
   ld hl,#tecla_pulsada
   ld (hl),a   

ret

   ;;grabar en RAM el FKC de la tecla pulsada detectada
   .arriba:
   ld hl,#tecla_pulsada ;cargar dirección etiqueta Tecla Pulsada para Basic
   ld a, (fkc_up)  ;extraer el valor FKC o Firmware Key Code
   ld (hl),a      ;grabar en RAM el FKC para Basic de la tecla "Arriba"
   ret

   .abajo:
   ld hl,#tecla_pulsada
   ld a, (fkc_down)
   ld (hl),a
   ret

   .izquierda:
   ld hl,#tecla_pulsada
   ld a, (fkc_left)
   ld (hl),a
   ret

   .derecha:
   ld hl,#tecla_pulsada
   ld a, (fkc_right)
   ld (hl),a
   ret

   .repetir:
   ld hl,#tecla_pulsada
   ld a, (fkc_rep)
   ld (hl),a
   ret

   .maspuntos:
   ld hl,#tecla_pulsada
   ld a, (fkc_more)
   ld (hl),a
   ret


;*********************************************
;          REDEFINICIÓN DEL TECLADO
;*********************************************

_leer_tecla_redef::
;;********************************* LEER UNA TECLA PARA REDEFINIRLA ***************************
;; Explora el teclado para ver si hay alguna tecla pulsada, grabar su Cte. Key Asm y su FKC 
;; ENTRADAS:
;; tecla_redef: donde le indico (1-4) cual tecla quiero redefinir (arr, aba, iz, der)
;; _constantes_keycode_asm: Array con las 80 Ctes. ASM de teclado para CPCTelera.
;; SALIDAS:
;; tecla_pulsada:: grabar el FKC (0-79) de cpct_isKeyPressed. Por defecto 128. Error 255.
;; tecla_pul_char:: .db 0 ;carácter ASCII asociado al FKC de la tecla pulsada
;; tecla_arriba::    .db #0,#0,#0 // En 1º y 2º bytes se graba Cte. Key Asm, en 3º el FKC
;; tecla_abajo::     .db #0,#0,#0
;; tecla_izquierda:: .db #0,#0,#0
;; tecla_derecha::   .db #0,#0,#0

   ;;Destroyed Register values: AF, BC, DE, HL
   call cpct_scanKeyboard_asm
   ;;cpct_isAnyKeyPressed_asm --- Input Parameters (2 Bytes)
   ;;(2B HL) Key Constant ASM. A 16-bit value containing a Matrix-Line(1B, L) and a BitMask(1B, H).
   ;;Return value (for Assembly, L=A=key_status)
   ;;<u8>	false (0, if not pressed) or true (>0, if pressed).  Take into account that true is not 1, 
   ;;but any non-0 number.
   ;;Flag Output Status: Z = 1 (Key NOT pressed) / 0 (Key pressed)
   ;;Destroyed Register values: A, B, HL
   call cpct_isAnyKeyPressed_asm
   or a	      ;no cambia A, pero activa los flags si A cambia, comprobando si la función devuelve 0.
   jp z,_salir ;mientras A devuelva 0, salir del escaneo de teclado y devolver control a Basic
      
   ld a,#0
   ld (#contador),a    ;inicializa var contador de índices del Array 0-79 (servirá como FKC)
   ld ix,#_constantes_keycode_asm ;cargar la dirección RAM de inicio del Array
   
   _prueba: 
   ;Preguntar a CPCTelera con cada uno de los 80 valores del Array, si coincide con su tecla pulsada  
   ;;cpct_isKeyPressed --- Checks if a concrete key is pressed or not.
   ;;Input Parameters (2 Bytes)
   ;;(2B HL) key	A 16-bit value containing a Matrix-Line(1B, L) and a BitMask(1B, H).
   ;;Return value (for Assembly, L=A=key_status)
   ;;<u8>	false (0, if not pressed) or true (>0, if pressed). Take into account that true is not 1, 
   ;;but any non-0 number.
   ;;Flag Output Status: Z = 1 (Key NOT pressed) / 0 (Key pressed) // C,*N* = 0 // H = 1
   ;;Destroyed Register values: A, D, BC, HL
        
   ld a, (ix)
   ld (#valorArray), a ;Se extrae 1 Cte. keycode asm de teclado, por ej. 8008, byte alto 1º

   ld__a_ix_mas_n  1 ;   ld a, (ix + 1)
   ld (#valorArray + 1), a  ;grabar byte bajo Cte. keycode asm del array
  
   ld hl, (valorArray)  ;meter en HL la Cte. Keycode Asm del array en formato Byte Alto-Byte Bajo
   call cpct_isKeyPressed_asm
   inc ix
   inc ix   
   jr nz, _pulsada ;si flag Z=0 es porque Cte. ASM Keyboard ha sido detectada por cpct_isKeyPressed_asm
   
   _nopulsada:
   ld a,(#contador)    ;usar var contador porque se destruyen los registros
   inc a
   ld (#contador),a 
   cp #80
   jr nz,_prueba
   _error:
   ld hl,#tecla_pulsada
   ld a,#0xFF
   ld (hl),a   ;grabar en la salida el código de error 255=&FF en vez de un FKC válido (0-79) 
   ret 

   _pulsada::
   ;Ha sido encontrada la tecla
   ld hl,#tecla_pulsada   ;dirección RAM de salida de la variable, que contiene 1 byte del FKC
   ld a,(#contador)       ;índice del array (0-79) coincidente con el FKC de la tecla pulsada
   ld (hl),a               ;grabarlo
   ld l,#0
   call 0xBB2A      ; KM GET TRANSLATE→Descubre qué símbolo o carácter asociado a tecla pulsada en ACU
   ld (tecla_pul_char),a 

   ;Preguntar a  tecla_redef cuál estoy redefiniendo (1-4)
   ld a, (tecla_redef)
   cp #1
   jr z,_up
   cp #2
   jr z,_dw
   cp #3
   jr z,_der
   cp #4
   jr z,_iz
   ret 

   _up: ;1
   ld hl, (valorArray)   ;cargo en hl la Cte. asm de teclado, por ej. 2002=shift
   ld (tecla_arriba), hl ;se graba 02 20 (byte bajo 1º), OK. 3er byte=FKC.
   ld bc, #tecla_arriba  ;dirección RAM 16 bits
   inc bc                ;acceso 2 byte
   inc bc                ;acceso 3er byte
   ld a,(#contador)      ;cargo FKC
   ld (bc),a             ;grabar el FKC en el 3er byte de tecla_arriba
   ret 

   _dw: ;2
   ld hl, (valorArray)  
   ld (tecla_abajo), hl 
   ld bc, #tecla_abajo  
   inc bc               
   inc bc               
   ld a,(#contador)     
   ld (bc),a            
   ret 

   _der: ;3
   ld hl, (valorArray)  
   ld (tecla_derecha), hl 
   ld bc, #tecla_derecha  
   inc bc               
   inc bc               
   ld a,(#contador)     
   ld (bc),a            
   ret

   _iz: ;4
   ld hl, (valorArray)  
   ld (tecla_izquierda), hl 
   ld bc, #tecla_izquierda  
   inc bc               
   inc bc               
   ld a,(#contador)     
   ld (bc),a            
   ret

  _salir:

ret 


_extraer_FKCs::
   ;;************ Extracción del 3er byte o FKC (Firmware Key Code) **********
   ;; Almacenarlo en variables fkc_up, fkc_down, fkc_left, fkc_right para que Basic pueda reconocerlas

   ld    hl,#tecla_arriba
   inc   hl ;en HL+2 debería encontrarse el 3er byte FKC 
   inc   hl
   ld    a,(hl)
   ld    hl,#fkc_up
   ld    (hl),a

   ld    hl,#tecla_abajo
   inc   hl ;en HL+2 debería encontrarse el 3er byte FKC 
   inc   hl
   ld    a,(hl)
   ld    hl,#fkc_down
   ld    (hl),a

   ld    hl,#tecla_izquierda
   inc   hl ;en HL+2 debería encontrarse el 3er byte FKC 
   inc   hl
   ld    a,(hl)
   ld    hl,#fkc_left
   ld    (hl),a

   ld    hl,#tecla_derecha
   inc   hl ;en HL+2 debería encontrarse el 3er byte FKC 
   inc   hl
   ld    a,(hl)
   ld    hl,#fkc_right
   ld    (hl),a

   ld    hl,#tecla_repetir
   inc   hl ;en HL+2 debería encontrarse el 3er byte FKC 
   inc   hl
   ld    a,(hl)
   ld    hl,#fkc_rep
   ld    (hl),a

   ld    hl,#tecla_maspuntos
   inc   hl ;en HL+2 debería encontrarse el 3er byte FKC 
   inc   hl
   ld    a,(hl)
   ld    hl,#fkc_more
   ld    (hl),a

ret



;*********************************************
;              TRANSFERENCIA RAM
;*********************************************
_transfer::
   ;;Transferir el SimBuffer.BIN desde &C000 a la posición reservada por el Bufer de Disco
   ;;tras terminar todas las operaciones de LOAD y así aprovechar sus 4KB en la franja de
   ;;&5000-&5FFF

   ld hl,#0xC000     ;fuente de los datos a copiar
   ld de,#0x5000     ;destino de los datos a copiar
   ld bc,#0xFB0      ;tamaño total de los datos a copiar, 
   ldir              ;copiar incrementando hl y de, decrementando bc y mientras bc no sea=0

ret


;*********************************************
;              PINTAR SPRITES VARIOS
;*********************************************

_pintar_spr_display::
   ;; Rótulo CPC + display 2 dígitos apagado, 18x31, 9bytes de ancho
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)     
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #36                  ;; C = x coordinate in bytes (0-79)
   ld    b, #87                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_display_sp
   ld    c, #8                   ;; ancho en bytes
   ld    b, #31                  ;; alto   
   call cpct_drawSprite_asm   

ret


_pintar_spr_num_display::
   ;; =======================================================================================
   ;; Rutina llamada desde BASIC: POSIC%=0: DIG%=9: CALL &XXXX,@POSIC%,@DIG% 
   ;; var DIG% proporciono el caracter (0-9) o sprite a imprimir.
   ;; var POSIC% proporciona la posición (0=unidades, 1=decenas) a imprimir dicho sprite.
   ;; =======================================================================================   
   ;obtener la dirección de la variable DIG% del comando Basic CALL
   ld__l_ix_mas_n #0x00 ;Macro de ld l, (ix + #0);  
   ld__h_ix_mas_n #0x01 ;Macro de ld h, (ix + #1);
   ld a, (hl) ; cargar el valor de la var Basic @DIG% en el Acumulador A
   ld (#digito_display), a ;almaceno en A el contenido de la var DIG% pasada desde Basic
   ;obtener la dirección de la variable POS% del comando Basic CALL
   ld__l_ix_mas_n #0x02 ;Macro de ld l, (ix + #0);  
   ld__h_ix_mas_n #0x03 ;Macro de ld h, (ix + #1);
   ld a, (hl) ; cargar el valor de la var Basic @POSIC% en el Acumulador A
   
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   cp #0
   jr z, _unidades
   
   _decenas:
   ld    c, #36                   ;; C = x coordinate in bytes (0-79)
   jp _calc_mem
   
   _unidades:
   ld    c, #40
   
   _calc_mem: 
   ld    b, #97                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL. Corrompe A 

   ;Fijar coordenadas donde pintar sprite (mis 10 sprites son recortes de 8x23pix = 4x23 bytes)
   ex   de, hl                    ;; DE = Pointer to start of the screen   
   ld    c, #4                   ;; ancho en bytes
   ld    b, #19                  ;; alto   

   ld a,(#digito_display) ; recuperar DIG% o dígito a mostrar en display

   ;Comparar el operando s con el Acumulador
   cp #0
   jp z,_cero
   cp #1
   jp z,_uno
   cp #2
   jp z,_dos
   cp #3
   jp z,_tres
   cp #4
   jp z,_cuatro
   cp #5
   jp z,_cinco
   cp #6
   jp z,_seis
   cp #7
   jp z,_siete
   cp #8
   jp z,_ocho
   cp #9
   jp z,_nueve

  _cero:   
   ld   hl, #_numseg_sp_00
   jp _imprimir
  _uno:
   ld   hl, #_numseg_sp_01
   jp _imprimir
  _dos:
   ld   hl, #_numseg_sp_02
   jp _imprimir
  _tres:
   ld   hl, #_numseg_sp_03
   jp _imprimir
  _cuatro:
   ld   hl, #_numseg_sp_04
   jp _imprimir
  _cinco:
   ld   hl, #_numseg_sp_05
   jp _imprimir
  _seis:
   ld   hl, #_numseg_sp_06
   jp _imprimir
  _siete:
   ld   hl, #_numseg_sp_07
   jp _imprimir
  _ocho:
   ld   hl, #_numseg_sp_08
   jp _imprimir
  _nueve:
   ld   hl, #_numseg_sp_09

  _imprimir:
   call cpct_drawSprite_asm   

ret


_pintar_spr_you::
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)     
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #36                  ;; C = x coordinate in bytes (0-79)
   ld    b, #87                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte, rótulo CPC + display apagado, 18x31, 9bytes de ancho)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, (you_sp)
   ld    c, #8                   ;; ancho en bytes
   ld    b, #7                  ;; alto   
   call cpct_drawSprite_asm
   
ret

;*********************************************
;               PINTAR SIMON
;*********************************************
_pintar_spr_simon::
   
   ;*********************** pintar gráfico o sprites ****************************
   ;Mode 1->1byte=4pix // Mode 0->1byte=2pix // Inicio coordenadas en bytes en esq. sup. izq.
   ;NOTA: imprimimos bytes, por tanto la imagen en Mode 1 tendrá que tener 1 resolución en 
   ;píxeles múltiplo de 4 o habrán líneas enteras o bordes sin pintar de otro color. 
   ;ld l,#0xAE
   ;call cpct_setBlendMode_asm

   ;; SPRITE 0      
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)
   ;; (2B DE) screen_start Pointer to the start of the screen (or a backbuffer)
   ;; (1B C ) x   [0-79] Byte-aligned column starting from 0 (x coordinate,
   ;; (1B B ) y   [0-199] row starting from 0 (y coordinate) in bytes)
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #14                   ;; C = x coordinate in bytes (0-79)
   ld    b, #0                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 
   ;;(2B HL) sprite Source Sprite Pointer (array with pixel data)
   ;;(2B DE) memory Destination video memory pointer
   ;;(1B C ) width  Sprite Width in bytes [1-63] (Beware, not in pixels!)
   ;;(1B B ) height Sprite Height in bytes (>0)
   ;pintar cuarto de imagen o sprite (mis 4 sprites son 4 cuadros de 52x100pix = 26x100 bytes)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_simon_sp_0
   ld    c, #26                   ;; ancho en bytes
   ld    b, #100                  ;; alto   
   call cpct_drawSprite_asm

   ;; SPRITE 1
    ;; Calculate a video-memory location for printing a string
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #40                   ;; C = x coordinate (16 = 0x10)
   ld    b, #0                   ;; B = y coordinate (24 = 0x18)   
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 
   ;pintar cuarto de imagen 
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_simon_sp_1
   ld    c, #26                  ;; ancho
   ld    b, #100                 ;; alto  
   call cpct_drawSprite_asm


   ;; SPRITE 2
    ;; Calculate a video-memory location for printing a string
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #14                   ;; C = x coordinate (16 = 0x10)
   ld    b, #100                   ;; B = y coordinate (24 = 0x18)   
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL   
   ;pintar cuarto de imagen 
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_simon_sp_2
   ld    c, #26                  ;; ancho
   ld    b, #100                 ;; alto
   call cpct_drawSprite_asm

   ;; sprite 3
    ;; Calculate a video-memory location for printing a string
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #40                   ;; C = x coordinate (16 = 0x10)
   ld    b, #100                   ;; B = y coordinate (24 = 0x18)   
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 
   ;pintar cuarto de imagen 
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_simon_sp_3
   ld    c, #26                  ;; ancho
   ld    b, #100                 ;; alto
   call cpct_drawSprite_asm

ret


_pintar_spr_pressanykey::
   ;; Rótulo sin CPC "Press Any Key" tapando todo el display 16x31, 8bytes de ancho
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)    

   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #36                  ;; C = x coordinate in bytes (0-79)
   ld    b, #87                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_pressanykey_sp 
   ld    c, #8                   ;; ancho en bytes
   ld    b, #31                  ;; alto   
   call cpct_drawSprite_asm   

ret 


_pintar_spr_youwin::
   ;; Rótulo "You WIN!!" tapando todo el display 16x31, 8bytes de ancho
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)    
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #36                  ;; C = x coordinate in bytes (0-79)
   ld    b, #87                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, (youwin_sp)
   ld    c, #8                   ;; ancho en bytes
   ld    b, #31                  ;; alto   
   call cpct_drawSprite_asm  

ret 


_pintar_spr_gameover::
   ;; Rótulo "Game Over" tapando todo el display 16x31, 8bytes de ancho   
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #36                  ;; C = x coordinate in bytes (0-79)
   ld    b, #87                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, (gameover_sp) 
   ld    c, #8                   ;; ancho en bytes
   ld    b, #31                  ;; alto   
   call cpct_drawSprite_asm   
   
ret 

_pintar_spr_nextlevel::
   ;; Rótulo "Next Level Press "L" Key" tapando todo el display 16x31, 8bytes de ancho   
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #36                  ;; C = x coordinate in bytes (0-79)
   ld    b, #87                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_nextlevel_sp 
   ld    c, #8                   ;; ancho en bytes
   ld    b, #31                  ;; alto   
   call cpct_drawSprite_asm   

ret 


;*********************************************
;              PINTAR Candelabro
;*********************************************
;;IZQUIERDO
_pintar_spr_candelabro_base_iz::
   ;; Candelabro (solo la base). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #10                  ;; C = x coordinate in bytes (0-79)
   ld    b, #136                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_0 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 

_pintar_spr_candelabro_f1_iz::
;; Candelabro (solo flama 1). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #10                  ;; C = x coordinate in bytes (0-79)
   ld    b, #122                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_1 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 

_pintar_spr_candelabro_f2_iz::
;; Candelabro (solo flama 1). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #10                  ;; C = x coordinate in bytes (0-79)
   ld    b, #122                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_2 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 

_pintar_spr_candelabro_f3_iz::
;; Candelabro (solo flama 1). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #10                  ;; C = x coordinate in bytes (0-79)
   ld    b, #122                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_3 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 


;;DERECHO
_pintar_spr_candelabro_base_der::
   ;; Candelabro (solo la base). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #69                  ;; C = x coordinate in bytes (0-79)
   ld    b, #136                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_0 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 

_pintar_spr_candelabro_f1_der::
;; Candelabro (solo flama 1). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #69                  ;; C = x coordinate in bytes (0-79)
   ld    b, #122                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_1 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 

_pintar_spr_candelabro_f2_der::
;; Candelabro (solo flama 2). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #69                  ;; C = x coordinate in bytes (0-79)
   ld    b, #122                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_2 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 

_pintar_spr_candelabro_f3_der::
;; Candelabro (solo flama 3). Cada recorte del tile es de 4x14 pix (2x14 bytes)
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #69                  ;; C = x coordinate in bytes (0-79)
   ld    b, #122                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte)
   ex   de, hl                    ;; DE = Pointer to start of the screen
   ld   hl, #_candelabro_sp_3 
   ld    c, #2                   ;; ancho en bytes
   ld    b, #14                  ;; alto   
   call cpct_drawSprite_asm  

ret 



;*********************************************
;         PINTAR Marco de los Menús
;*********************************************

_pintar_spr_marcomenus::
 ;;PINTAR LOS MARCOS CUADRADOS DE LOS MENÚS Y TABLA PUNTOS. SON 4 TORNILLOS Y FILAS DE REMACHES
 ;; Salvar antes los Registros del uP   
 ;; TORNILLO SUP IZQ
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #12                  ;; C = x coordinate in bytes (0-79)
   ld    b, #6                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte 16x16pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_marcomenus_sp_0    ;; TORNILLO SUP IZQ
   ld    c, #4                   ;; ancho en bytes
   ld    b, #16                  ;; alto   
   call cpct_drawSprite_asm
 
 ;; REMACHE HORIZONTAL SUPERIOR
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)      
   ld    hl,#mem_ptr
   ld    a, #16            ;;bucle FOR a=16 (coordenada X=16 de 0-79)
   ld    (hl),a            ;;guardar A en mem_ptr, porque se destruye
   
   _bucle_remhor: 
   ld   de, #0xC000  
   ld    c, a                    ;; C = x coordinate in bytes (0-79)
   ld    b, #6                   ;; B = y coordinate in bytes (0-199)
   call  cpct_getScreenPtr_asm   ;; Calculate video memory location and return it in HL    
   call  _pintar_remache_hor
   ld    hl,#mem_ptr
   ld    a,(hl)      ;recuperar A
   add   a, #4       ;sprite=4x16bytes, para imprimir a lo ancho incremento de 4 en 4
   ld    (hl),a      ;guardar A
   cp    #64         ;TO 64 : NEXT ->restar operando 64 del A
   jp nz,_bucle_remhor

 ;; REMACHE HORIZONTAL INFERIOR
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)      
   ld    hl,#mem_ptr
   ld    a, #16            ;;bucle FOR a=16 (coordenada X=16 de 0-79)
   ld    (hl),a            ;;guardar A en mem_ptr, porque se destruye
   
   _bucle_remhorinf: 
   ld   de, #0xC000  
   ld    c, a                    ;; C = x coordinate in bytes (0-79)
   ld    b, #182                   ;; B = y coordinate in bytes (0-199)
   call  cpct_getScreenPtr_asm   ;; Calculate video memory location and return it in HL    
   call  _pintar_remache_hor
   ld    hl,#mem_ptr
   ld    a,(hl)      ;recuperar A
   add   a, #4       ;sprite=4x16bytes, para imprimir a lo ancho incremento de 4 en 4
   ld    (hl),a      ;guardar A
   cp    #64         ;TO 64 : NEXT ->restar operando 64 del A
   jp nz,_bucle_remhorinf   

 ;; TORNILLO SUP DER
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #64                  ;; C = x coordinate in bytes (0-79)
   ld    b, #6                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte 16x16pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_marcomenus_sp_2    ;; TORNILLO SUP DER
   ld    c, #4                   ;; ancho en bytes
   ld    b, #16                  ;; alto   
   call cpct_drawSprite_asm


 ;; COLUMNA REMACHE VERTICAL IZQ
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)     
   ld    hl,#mem_ptr
   ld    a, #22            ;;bucle FOR a=24 TO 184 ;; B = y coordinate in bytes (0-199)
   ld    (hl),a            ;;guardar A en mem_ptr, porque se destruye
   
   _bucle_remver: 
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #12                  ;; C = x coordinate in bytes (0-79)
   ld    b, a                    ;; B = y coordinate in bytes (0-199)
   call  cpct_getScreenPtr_asm   ;; Calculate video memory location and return it in HL 
   call  _pintar_remache_vert
   ld    hl,#mem_ptr
   ld    a,(hl)      ;recuperar A
   add   a, #16       ;sprite=4x16bytes, para imprimir a lo alto incremento de 16 en 16
   ld    (hl),a      ;guardar A
   cp    #182         ;TO 184 : NEXT ->restar operando 184 del A
   jp nz,_bucle_remver 


 ;; COLUMNA REMACHE VERTICAL DER
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)     
   ld    hl,#mem_ptr
   ld    a, #22            ;;bucle FOR a=24 TO 184  ;; B = y coordinate in bytes (0-199)
   ld    (hl),a            ;;guardar A en mem_ptr, porque se destruye
   
   _bucle_remverder: 
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #64                  ;; C = x coordinate in bytes (0-79)
   ld    b, a                    ;; B = y coordinate in bytes (0-199)
   call  cpct_getScreenPtr_asm   ;; Calculate video memory location and return it in HL 
   call  _pintar_remache_vert
   ld    hl,#mem_ptr
   ld    a,(hl)      ;recuperar A
   add   a, #16       ;sprite=4x16bytes, para imprimir a lo alto incremento de 16 en 16
   ld    (hl),a      ;guardar A
   cp    #182         ;TO 184 : NEXT ->restar operando 184 del A
   jp nz,_bucle_remverder  

;; TORNILLO INF DER
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #12                  ;; C = x coordinate in bytes (0-79)
   ld    b, #182                 ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 
   ;pintar sprite (1 solo recorte 16x16pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_marcomenus_sp_5    ;; TORNILLO INF DER
   ld    c, #4                   ;; ancho en bytes
   ld    b, #16                  ;; alto   
   call cpct_drawSprite_asm

;; TORNILLO INF IZQ
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #64                  ;; C = x coordinate in bytes (0-79)
   ld    b, #182                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 
   ;pintar sprite (1 solo recorte 16x16pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_marcomenus_sp_4    ;; TORNILLO INF DER
   ld    c, #4                   ;; ancho en bytes
   ld    b, #16                  ;; alto   
   call cpct_drawSprite_asm

;; RÓTULO SUP MEMTRAINER 
   ;; Calculate a video-memory location for printing a sprite. Returns a byte-pointer 
   ;; to a screen memory location, given its X, Y coordinates.  (in bytes, NOT in pixels!)  
   ld   de, #0xC000              ;; DE = Pointer to start of the screen
   ld    c, #28                  ;; C = x coordinate in bytes (0-79)
   ld    b, #0                   ;; B = y coordinate in bytes (0-199)  
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL 

   ;pintar sprite (1 solo recorte 100x26pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
   ex   de, hl                   ;; DE = Pointer to start of the screen
   ld   hl, #_rotulomemtrainer_sp     
   ld    c, #25                   ;; ancho en bytes
   ld    b, #26                  ;; alto   
   call cpct_drawSprite_asm 

ret 

   _pintar_remache_hor:
      ;pintar sprite (1 solo recorte 16x16pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
      ex   de, hl                   ;; DE = Pointer to start of the screen
      ld   hl, #_marcomenus_sp_1    ;; TORNILLO SUP IZQ
      ld    c, #4                   ;; ancho en bytes
      ld    b, #16                  ;; alto   
      call cpct_drawSprite_asm
   ret

   _pintar_remache_vert:
      ;pintar sprite (1 solo recorte 16x16pix) ;Mode 1->1byte=4pix // ancho 16pix=4 bytes
      ex   de, hl                   ;; DE = Pointer to start of the screen
      ld   hl, #_marcomenus_sp_3    ;; TORNILLO SUP IZQ
      ld    c, #4                   ;; ancho en bytes
      ld    b, #16                  ;; alto   
      call cpct_drawSprite_asm
   ret





